module Css exposing
  ( style, stylesheet
  , rule, dec
  , Property, Value, Declaration, Selector, Rule
  )

{-| A library to create and use css stylesheets in Elm.
# Type Definitions
@docs Property, Value, Declaration, Selector, Rule
# The Style Tag
@docs style
# Helpers
@docs stylesheet, dec, rule
-}

import Html exposing (node, Attribute, Html, text)
import String

{-| It should be a valid CSS Property (as String).

    -- Example

    "background-color" : Property
-}
type alias Property = String


{-| It should be a valid CSS Value (as String).

    -- Example

    "1px solid black" : Value
-}
type alias Value = String


{-| A Declaration consists of (is a tuple of) a Property and a Value.
It should be a valid CSS Declaration.

    -- Example

    blueBgc = ("background-color", "lightblue") -- Declaration (Property, Value)
-}
type alias Declaration = (Property, Value)


{-| It should be a valid CSS Selector (as String).

    -- Example

    "div.foo > span.bar, #baz, [disabled]" : Selector
-}
type alias Selector = String


{-| A Rule consists of (is a tuple of) a Selector and List of Declarations.
It should be a valid CSS Rule.

    -- Example

    alertText = ("color", "red")                -- Declaration (Property, Value)
    alertBg = ("background-color", "lightpink") -- Declaration (Property, Value)

    alertRule =
      ( "span.has-alert"
      , [ alertText
        , alertBg
        ]
      )                                         -- Rule
-}
type alias Rule = (Selector, List Declaration)


{-| Converts a Property to equivalent CSS Property.
-}
cssProperty : Property -> String
cssProperty property =
  property


{-| Converts a Value to equivalent CSS Value.
-}
cssValue : Value -> String
cssValue value =
  value


{-| Converts a Selector to equivalent CSS Selector.
-}
cssSelector : Selector -> String
cssSelector selector =
  selector


{-| Converts a Declaration to equivalent CSS Declaration.

    -- Example

    cssDeclaration ("z-index", "5") == "z-index:5;"
-}
cssDeclaration : Declaration -> String
cssDeclaration (property, value) =
  cssProperty property ++ ":" ++ cssValue value ++ ";"


{-| Converts a Rule to equivalent CSS Rule.

    -- Example

    cssRule
      ("h1.banner"
      , [ ("font-family", "'Comic Sans MS',fantasy")
        , ("color", "puple !important")
        ]
      )

    ==

    "h1.banner{font-family:'Comic Sans MS',fantasy;color:purple !important;}"
-}
cssRule : Rule -> String
cssRule (selector, declarations) =
  let
    cssDeclarations =
      List.foldl
        (\declaration cssString ->  cssString ++ cssDeclaration declaration)
        ""
        declarations
  in
    cssSelector selector ++ "{" ++ cssDeclarations ++ "}"


{-| Constructor for Rule. (,) can be used instead.
-}
rule : Selector -> List Declaration -> Rule
rule =
  (,)


{-| Constructor for Declaration. (,) can be used instead.
-}
dec : Property -> Value -> Declaration
dec =
  (,)


{-| Html style tag (for internal styles).

    -- Example

    style [] []

    -- <style></style>

    style [] [text "body { background-color : blue }"]

    --  <style>
    --    body {
    --      backdround-color : blue
    --    }
    --  </style>
-}
style : List (Attribute msg) -> List (Html msg) -> Html msg
style =
  node "style"


{-| Helper for style tag.

    -- Example

    themedStyleSheet primaryColor bgColor =
      stylesheet
      [ rule "body"
        [ dec "background-color" bgColor
        ]
      , rule "button"
        [ dec "color" primaryColor
        ]
      ]
-}
stylesheet : List Rule -> Html msg
stylesheet rules =
  style [] (List.map (cssRule >> text) rules)